/*******************************************************************

	vodomerka_functions.c
	Jakub Michnik (C) 2009.  All rights reserved.

	This program is used with RCM5400W series controllers
	with Vodomerka 2.0.

	Description
	===========
	various function

*******************************************************************/
/*** BeginHeader  */
#ifndef _VODOMERKA_FUNCTIONS
#define _VODOMERKA_FUNCTIONS
/*** EndHeader  */


/*** BeginHeader meas_struct_load */
int meas_struct_load(void);
/*** EndHeader  */

int meas_struct_load(void)
{
//vraci 1 kdyz se to povede
   FATfile my_file;  // fat file structure
   int length, rc;

   log_add("Called meas_struct_load\n");
   length=sizeof(meas_week);
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "S_W_MAR.TXT",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_READONLY,	// Create the file if it does not exist.
                 &my_file,		// Fill in this structure with file details
                 0		// Number of bytes to allocate.
                );
	if (rc < 0) {
   	sprintf( &last_err, "fat_Open() in meas_struct_load failed with return code %d\n", rc);
      log_add(&last_err);
   	printf("fat_Open() struct_load failed with return code %d\n", rc);
		return(0);
   }
   rc = fat_Read( &my_file, &meas_week, length);
   if (rc < 0) {
      sprintf( &last_err, "fat_Read() in meas_struct_load failed with return code %d\n", rc);
      log_add(&last_err);
   	printf("fat_Read() struct_load  failed with return code %d\n", rc);
      return(0);
   }
   if(rc != length){
      sprintf( &last_err, "meas_struct_load loaded only %d bytes\n", rc);
      log_add(&last_err);
   	printf("struct_load loaded only %d bytes\n", rc );
   }
  	rc = fat_Close(&my_file);
   if (rc < 0) {
   	sprintf( &last_err, "fat_Close() in meas_struct_load failed with return code %d\n", rc);
      log_add(&last_err);
   	printf(&last_err);
		return(0);
   }
   else
   	log_add("meas_struct_load OK\n");
   	printf("struct_load succesfull\n");
   return(1);
}


/*** BeginHeader meas_struct_save */
int meas_struct_save(void);
/*** EndHeader  */

int meas_struct_save(void)
{
   FATfile my_file;  // fat file structure
   int prealloc, rc;

   prealloc=sizeof(meas_week);
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "S_W_MAR.TXT",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_CREATE,	// Create the file if it does not exist.
                 &my_file,		// Fill in this structure with file details
                 0		// Number of bytes to allocate.
                );
	if (rc < 0) {
   	sprintf( &last_err, "fat_Open() in meas_struct_save failed with return code %d\n", rc);
      log_add(&last_err);
	   fix_err = 1;
   	printf("fat_Open() failed with return code %d\n", rc);
      format();
		return(0);
   }
   rc = fat_Write( &my_file, &meas_week, prealloc );
   if (rc < 0) {
      sprintf( &last_err, "fat_Write() in meas_struct_save failed with return code %d\n", rc);
      log_add(&last_err);
	   fix_err = 1;
   	printf("fat_Write() failed with return code %d\n", rc);
      format();
      return(0);
   }
  	rc = fat_Close(&my_file);
   if (rc < 0) {
      sprintf( &last_err, "fat_Close() in meas_struct_save failed with return code %d\n", rc);
      log_add(&last_err);
	   fix_err = 1;
   	printf("fat_Close() failed with return code %d\n", rc);
      format();
		return(0);
   }
   else{
   	//log_add("meas_struct_save OK\n");
   }
}


/*** BeginHeader meas_struct_init */
void meas_struct_init(void);
/*** EndHeader  */

void meas_struct_init(void)
{
	int i;
   float a,b;
   tm_typ time;

   log_add("Called meas_struct_init\n");
   b = SEC_TIMER - 1800; // minus pulhodiny
   for(i=0;i<1008;i++)
   {
      meas_week.water[i]=-1;
      //meas_week.water[i]= sin(deg(i*0.89285/2))*1000.0 + 1100;
   	meas_week.temp[i]=0;
      a=(1007-i)*600.0;
   	meas_week.time[i]=b-a;
   }
   meas_week.last_write=1007;
   printf("meas_struct_init\n") ;


}


/*** BeginHeader meas_struct_update */
void meas_struct_update(void);
/*** EndHeader  */

void meas_struct_update(void)
{
	tm_typ time2;
   int i,j;             // i: posun od last_write. 0 zapisuje na last_write
   long delta_time;

// week UPDATE
   delta_time = SEC_TIMER - meas_week.time[meas_week.last_write];
	mktm(&time2, delta_time);


   //pokud od posledniho mereni uplynulo vice nez tyden, nebo je casovy
   //rozdil zaporny, provodede se inicializace struktury meas_week
   if( ((time2.tm_mon-1 + time2.tm_year-80) > 0) | ( time2.tm_mday > 7)
   															 | ( delta_time < 0  ))
   {
   	printf("Delta_time error, meas_struct_init called");
      log_add("delta_time error in meas_struct_update \n");
      meas_struct_init();
      i=1;
   }
   else
   {
	   i = (time2.tm_mday-1)*144 + time2.tm_hour*6 + time2.tm_min/10.0;
#ifdef MY_DEBUG_10
		// pokud byla casova zmena pod 10min, zapise se na nejblizsi prvek
      if(i==0) i=1;
#endif

      i += meas_week.last_write;
   }
      if(i>1007)
      {
      	for(j=meas_week.last_write+1;j<i;j++) //nulovani do konce struktury
         {
         	if (j>1007)
            	break;
            meas_week.water[j]=-1;
            meas_week.temp[j]=0;
            meas_week.time[j]=0;
         }
         i-=1008;
         for(j=0;j<i;j++)
         {
            meas_week.water[j]=-1;
            meas_week.temp[j]=0;
            meas_week.time[j]=0;
         }
      }
      else
      {
         for(j=meas_week.last_write+1;j<i;j++)
         {
            meas_week.water[j]=-1;
            meas_week.temp[j]=0;
            meas_week.time[j]=0;
         }
      }
      meas_week.time[i] = SEC_TIMER;
	   meas_week.water[i] = meas_values[0];
   	meas_week.temp[i]  = meas_values[4];
	   meas_week.last_write = i;
}

/*** BeginHeader after_reset */
void after_reset(void);
/*** EndHeader  */

void after_reset(void)
{
	//load structures;
   //if not initiate
   tm_typ char_time, time;

   if(SEC_TIMER < 640800)
   {
   	printf("Wrong time!!!\n");
      log_add("Wrong time\n");
   	char_time.tm_sec = 0;
		char_time.tm_min = 0;
      char_time.tm_hour = 0;
      char_time.tm_mday = 1;
      char_time.tm_mon = 1;
      char_time.tm_year = 110;
		tm_wr(&char_time);
      SEC_TIMER = read_rtc();
      mktm(&time, SEC_TIMER);
      printf("------ %d:%d:%d  %d.%d.%d ------ \n",  time.tm_hour, time.tm_min,
      				time.tm_sec,time.tm_mday, time.tm_mon, time.tm_year+1900);
      log_add("<-- is current time\n");
      meas_struct_init();
   }

   if( !meas_struct_load() )
 		meas_struct_init();
   if( !opt_load() )
   	opt_init();


	printf("last write %d\n", meas_week.last_write ) ;
   mktm(&time, meas_week.time[meas_week.last_write]);
   printf("last write time %d:%d:%d  %d.%d.%d ------ \n",  time.tm_hour, time.tm_min,
      				time.tm_sec,time.tm_mday, time.tm_mon, time.tm_year+1900);
   mktm(&time, meas_week.time[0]);
   printf("index 0 time %d:%d:%d  %d.%d.%d ------ \n",  time.tm_hour, time.tm_min,
      				time.tm_sec,time.tm_mday, time.tm_mon, time.tm_year+1900);
   mktm(&time, meas_week.time[1007]);
   printf("index 1007 time %d:%d:%d  %d.%d.%d ------ \n\n",  time.tm_hour, time.tm_min,
      				time.tm_sec,time.tm_mday, time.tm_mon, time.tm_year+1900);

}



/*** BeginHeader xml_24_create  */
void xml_24_create(void);
/*** EndHeader  */

void xml_24_create(void)
{
	// create XML for 24h graph
   FATfile xml_file;  // fat file structure
   char buf[100], time_buf[6];
   int rc, buflength,j,i,k;
   long prealloc;
   tm_typ time;
   int value,a;


   rc = fat_Delete(partition, FAT_FILE, "W_24_MAR.XML");
   prealloc = 0;
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "W_24_MAR.XML",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_CREATE,	// Create the file if it does not exist.
                 &xml_file,		// Fill in this structure with file details
                 &prealloc		// Number of bytes to allocate.
                );
	if (rc < 0) {
      sprintf( &last_err, "fat_Open() in xml_24_create failed with return code %d\n", rc);
      log_add(&last_err);
	   fix_err = 1;
   	printf("fat_Open() failed with return code %d\n", rc);
      format();
	  	return;
   }

   rc=0;

   sprintf(&buf, "<chart_data>\n<row>\n<null/>\n"); buflength = 27;
	rc = fat_Write( &xml_file, &buf, buflength );
   j=meas_week.last_write - 143;   //urcite 144?nema byt 143?
   if(j<0)
   	k=1008+j;
   else
   	k=j;
   i=0;
   while(i<144)
   {
     if(k>=1008)
      	k=0;
      sprintf(&buf, "<string>"); buflength = 8;
		rc = fat_Write( &xml_file, &buf, buflength );
      mktm(&time, meas_week.time[meas_week.last_write] - (143-i)*600.0 );
     	//mktm(&time, meas_week.time[k]);
      a = time.tm_hour;
      if( (time.tm_min < 10) & ((a==0)|(a==3)|(a==6)|(a==9)|(a==12)
      		|(a==15)|(a==18)|(a==21)) ) //& (meas_week.time[k] != 0) )
      {
      	buf[0] = time.tm_hour/10.0;
			buf[1] = time.tm_hour-10*buf[0] + 48;
         buf[0] += 48;
         buf[2] = ':';
         buf[3] = '0';
         buf[4] = '0';
         buflength = 5;
			rc += fat_Write( &xml_file, &buf, buflength );
      }
      sprintf(&buf, "</string>\n"); buflength = 10;
		rc = fat_Write( &xml_file, &buf, buflength );
      if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_24_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
      }
      k++;
      i++;
   }
   sprintf(&buf, "</row>\n"); buflength = 7;
	rc = fat_Write( &xml_file, &buf, buflength );

   /////////////////////////////////////////////////////////////////

   sprintf(&buf, "<row>\n<string>Marklovice</string>\n"); buflength = 34;
	rc = fat_Write( &xml_file, &buf, buflength );
   if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_24_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
   }
   if(j<0)
   	k=1008+j;
   else
   	k=j;
   i=0;
   while(i<144)
   {
     if(k>=1008)
      	k=0;
     if(( meas_week.time[k] != 0) & (meas_week.time[k] != -1) )
     {
     		mktm(&time, meas_week.time[k]);
      	time_buf[0] = time.tm_hour/10.0;
			time_buf[1] = time.tm_hour-10*time_buf[0] + 48;
      	time_buf[0] += 48;
      	time_buf[2] = ':';
      	time_buf[3] = time.tm_min/10.0;
      	time_buf[4] = time.tm_min-10*time_buf[3] + 48;
      	time_buf[3] += 48;
      	time_buf[5] = 0;
      	buflength = 6;

     		sprintf(&buf, "<number tooltip=\'%s\'>", &time_buf);
      	buflength = 24;
			rc = fat_Write( &xml_file, &buf, buflength );
      	value = meas_week.water[k]/10.0; // value in cm
      	itoa(value, &buf);
			rc = fat_Write( &xml_file, &buf, strlen(buf) );
         sprintf(&buf, "</number>\n"); buflength = 10;
			rc = fat_Write( &xml_file, &buf, buflength );
         if (rc < 0){
      		sprintf( &last_err, "fat_Write() in xml_24_create failed with return code %d\n", rc);
      		log_add(&last_err);
	   		fix_err = 1;
   			printf("fat_Write() failed with return code %d\n", rc);
         	format();
         	return;
      	}
     }
     else
     {
         sprintf(&buf, "<null/>\n"); buflength = 8;
			rc = fat_Write( &xml_file, &buf, buflength );
         if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_24_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
      }
     }

      k++;
      i++;
   }

   sprintf(&buf, "</row>\n</chart_data>"); buflength = 20;
	rc = fat_Write( &xml_file, &buf, buflength );
   if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_24_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
      }
  	rc = fat_Close(&xml_file);
   if (rc < 0){
      	sprintf( &last_err, "fat_Close() in xml_24_create failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Write() failed with return code %d\n", rc);
        // format();
         return;
   }
}


/*** BeginHeader xml_week_create  */
void xml_week_create(void);
/*** EndHeader  */

void xml_week_create(void)
{
	// create XML for week graph
   FATfile xml_file;  // fat file structure
   char buf[100], time_buf[6];
   int rc, buflength,j,i,k;
   long prealloc;
   tm_typ time;
   int value,a;

   rc = fat_Delete(partition, FAT_FILE, "W_W_MAR.XML");
   prealloc = 0;
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "W_W_MAR.XML",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_CREATE,	// Create the file if it does not exist.
                 &xml_file,		// Fill in this structure with file details
                 &prealloc		// Number of bytes to allocate.
                );
   if (rc < 0){
      	sprintf( &last_err, "fat_Open() in xml_week_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Open() failed with return code %d\n", rc);
         format();
         return;
   }

   rc=0;

   sprintf(&buf, "<chart_data>\n<row>\n<null/>\n"); buflength = 27;
	rc = fat_Write( &xml_file, &buf, buflength );
   j=meas_week.last_write - 1007;
   if(j<0)
   	k=1008+j;
   else
   	k=j;
   i=0;
   while(i<1008)
   {
     if(k>=1008)
      	k=0;
      sprintf(&buf, "<string>"); buflength = 8;
		rc = fat_Write( &xml_file, &buf, buflength );
      if (rc < 0)
   		printf("fat_Write() failed with return code %d\n", rc);
     	mktm(&time, meas_week.time[meas_week.last_write] - (1007-i)*600.0 );

      if( (time.tm_min < 9) & (time.tm_hour==0) )
      {
      	switch(time.tm_wday){
          	case 0:	sprintf(&buf,"        Nedele"); break;
				case 1:	sprintf(&buf,"        Pondeli"); break;
				case 2:	sprintf(&buf,"        Utery"); break;
				case 3:	sprintf(&buf,"        Streda"); break;
				case 4:	sprintf(&buf,"        Ctvrtek"); break;
				case 5:	sprintf(&buf,"        Patek"); break;
				case 6:	sprintf(&buf,"        Sobota"); break;
            default: sprintf(&buf,"error wday"); break;
         }
		rc = fat_Write( &xml_file, &buf, strlen(buf) );
      }

      sprintf(&buf, "</string>\n"); buflength = 10;
		rc = fat_Write( &xml_file, &buf, buflength );
      if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_week_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
   	}
		k++;
      i++;
   }
   sprintf(&buf, "</row>\n"); buflength = 7;
	rc = fat_Write( &xml_file, &buf, buflength );

   /////////////////////////////////////////////////////////////////

   sprintf(&buf, "<row>\n<string>Marklovice</string>\n"); buflength = 34;
	rc = fat_Write( &xml_file, &buf, buflength );
   if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_week_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
   }
   if(j<0)
   	k=1008+j;
   else
   	k=j;
   i=0;
   while(i<1008)
   {
     if(k>=1008)
      	k=0;
     if(( meas_week.time[k] != 0) & (meas_week.time[k] != -1) )
     {
     		mktm(&time, meas_week.time[k]);
      	time_buf[0] = time.tm_hour/10.0;
			time_buf[1] = time.tm_hour-10*time_buf[0] + 48;
      	time_buf[0] += 48;
      	time_buf[2] = ':';
      	time_buf[3] = time.tm_min/10.0;
      	time_buf[4] = time.tm_min-10*time_buf[3] + 48;
      	time_buf[3] += 48;
      	time_buf[5] = 0;
      	buflength = 6;

     		sprintf(&buf, "<number tooltip=\'%s\'>", &time_buf);
      	buflength = 24;
			rc = fat_Write( &xml_file, &buf, buflength );
      	value = meas_week.water[k]/10.0; // value in cm
      	itoa(value, &buf);
			rc = fat_Write( &xml_file, &buf, strlen(buf) );
         sprintf(&buf, "</number>\n"); buflength = 10;
			rc = fat_Write( &xml_file, &buf, buflength );
         if (rc < 0){
      		sprintf( &last_err, "fat_Write() in xml_week_create failed with return code %d\n", rc);
      		log_add(&last_err);
	   		fix_err = 1;
   			printf("fat_Write() failed with return code %d\n", rc);
         	format();
         	return;
   		}
     }
     else
     {
         sprintf(&buf, "<null/>\n"); buflength = 8;
			rc = fat_Write( &xml_file, &buf, buflength );
         if (rc < 0){
      		sprintf( &last_err, "fat_Write() in xml_week_create failed with return code %d\n", rc);
        		log_add(&last_err);
	     		fix_err = 1;
   	  		printf("fat_Write() failed with return code %d\n", rc);
         	format();
        		return;
   		}
     }

      k++;
      i++;
   }

   sprintf(&buf, "</row>\n</chart_data>"); buflength = 20;
	rc = fat_Write( &xml_file, &buf, buflength );
  	if (rc < 0){
      	sprintf( &last_err, "fat_Write() in xml_week_create failed with return code %d\n", rc);
      	log_add(&last_err);
	   	fix_err = 1;
   		printf("fat_Write() failed with return code %d\n", rc);
         format();
         return;
   }
  	rc = fat_Close(&xml_file);
   if (rc < 0){
      	sprintf( &last_err, "fat_Close() in xml_week_create failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Close() failed with return code %d\n", rc);
         //format();
         return;
   }
}



/*** BeginHeader other_init  */
void other_init(void);
/*** EndHeader  */

void other_init(void)
{
//FAT variables
   	long part_free_size;
   	int part_free_size_kB;
//DHCP
   	long ipaddr;
   	char ipaddr_string[16];
//FTP
      int admin_user;
      char buf[20];
//other
     	char i,j,rc;

// FAT
   rc = fat_AutoMount(FDDF_USE_DEFAULT);
   partition = NULL;
	for (i = 0; i < num_fat_devices * FAT_MAX_PARTITIONS; ++i)
   {
		if ((partition = fat_part_mounted[i]) != NULL)
      {
			break; // found a mounted partition, so use it
		}
	}
   if(partition == NULL)
   	rc= (rc < 0) ? rc : -ENOPART;
   else
   {
    	printf("fat_AutoMount() succeeded with return code %d.\n", rc);
      rc=0;
   }
	if (rc < 0)
   {
   	if (rc == -EUNFORMAT)
      	printf("Device not Formatted, Please run Fmt_Device.c\n");
      else
	   	printf("fat_AutoMount() failed with return code %d.\n", rc);
   }
   part_free_size = fat_Free(partition); // proc to nejede na jednom radku???
   part_free_size *= FAT_SECSIZE;
   part_free_size_kB = (int)(part_free_size/1000.0) ;
   if(part_free_size < 1000)
   {
   	sprintf(&last_err, "PROGRAM START; ---; Na disku A je mene nez 1KB volneho mista, called format partition A\n");
      fix_err = 1;
		printf("Malo mista, called format partition A");
      format();
      //restart!!
   }
   else{

   		sprintf(&last_err, "Na disku je: %d kB volneho mista\n", part_free_size_kB);
         log_add("\n\n-------------\r\n");
		   log_add("PROGRAM START\r\n");
         log_add(&last_err);
   }

// end of FAT


// begin LAN DHCP connecting
   printf ("configuring interface for DHCP\n");
   ifconfig(IF_WIFI0, IFS_DHCP, 1, IFS_END);
   sock_init();
   while (ifpending(IF_DEFAULT) == IF_COMING_UP) // dodelat nejaky timeout?
         tcp_tick(NULL);
   ifconfig(IF_WIFI0,IFG_IPADDR,&ipaddr,IFS_END);
   printf("IP Address: %s - responding to pings\n",inet_ntoa(ipaddr_string,ipaddr));
   sprintf(&last_err, "IP Address: %s - responding to pings\n",inet_ntoa(ipaddr_string,ipaddr));
   log_add(&last_err);
   log_add("DHCP config OK\n");
// end LAN DHCP connecting

// begin FTP server setup
 	 	if (sspec_automount(SSPEC_MOUNT_ANY, NULL, NULL, NULL)) {
   		printf("Failed to setup FAT.  Please refer to FAT samples/documentation.\n");
   		exit(1);
   	}
      // Set up foo user.
		admin_user = sauth_adduser("admin", "admin", SERVER_FTP);
		sauth_setusermask(admin_user, ADMIN_GROUP, NULL);
		sauth_setwriteaccess(admin_user, SERVER_FTP);	  // This allows write access
		//sauth_setusermask(foo_user, FOO_GROUP | ANON_GROUP, NULL);
   	ftp_init(NULL); // use default handlers
		tcp_reserveport(FTP_CMDPORT);	// Port 21
		printf("Ready: FTP to %s\n", inet_ntoa(buf, MY_ADDR(IF_DEFAULT)));
      sprintf( &last_err, "Ready: FTP to %s\n", inet_ntoa(buf, MY_ADDR(IF_DEFAULT)));
      log_add(&last_err);
      log_add("FTP config OK\n");

// end FTP server setup

//begin HTTP server setup

   /* init FORM searchable names - must init ALL FORMSpec structs!    */
   FORMmeasure[0].name = "measure_interval";
   FORMmeasure[1].name = "k";
   FORMmeasure[2].name = "q";
   FORMtime[0].name = "year";
   FORMtime[1].name = "month";
   FORMtime[2].name = "day";
   FORMtime[3].name = "hour";
   FORMtime[4].name = "minute";
   FORMtime[5].name = "second";

   http_init();
 	tcp_reserveport(80);
   log_add("HTTP config OK\n");
//end HTTP server setup

}


/*** BeginHeader opt_save */
void opt_save(void);
/*** EndHeader  */

void opt_save(void)
{
   FATfile my_file;  // fat file structure
   int prealloc, rc;

   prealloc=sizeof(opt);
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "OPT_MAR.OPT",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_CREATE,	// Create the file if it does not exist.
                 &my_file,		// Fill in this structure with file details
                 &prealloc		// Number of bytes to allocate.
                );
   if (rc < 0){
      	sprintf( &last_err, "fat_Open() in opt_save failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Open() failed with return code %d\n", rc);
         return;
   }
   rc = fat_Write( &my_file, &opt, prealloc );
   if (rc < 0){
      	sprintf( &last_err, "fat_Read() in opt_save failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Open() failed with return code %d\n", rc);
         return;
   }
  	rc = fat_Close(&my_file);
   if (rc < 0){
      	sprintf( &last_err, "fat_Close() in opt_save failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Close() failed with return code %d\n", rc);
         return;
   }
}


/*** BeginHeader opt_load */
int opt_load(void);
/*** EndHeader  */

int opt_load(void)
{
//vraci 1 kdyz se to povede
   FATfile my_file;  // fat file structure
   int length, rc;

   length=sizeof(meas_week);
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "OPT_MAR.OPT",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_READONLY,	// Create the file if it does not exist.
                 &my_file,		// Fill in this structure with file details
                 0		// Number of bytes to allocate.
                );
   if (rc < 0){
      	sprintf( &last_err, "fat_Open() in opt_load failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Open() failed with return code %d\n", rc);
         return(0);
   }
   rc = fat_Read( &my_file, &opt, length);
   if (rc < 0){
      	sprintf( &last_err, "fat_Read() in opt_load failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Open() failed with return code %d\n", rc);
         return(0);
   }

  	rc = fat_Close(&my_file);
   if (rc < 0){
      	sprintf( &last_err, "fat_Close() in opt_load failed with return code %d\n", rc);
      	log_add(&last_err);
   		printf("fat_Close() failed with return code %d\n", rc);
         return(0);
   }
   else{
   	printf("opt_load succesfull\n");
      log_add("opt_load OK\n");
   }
   return(1);
}


/*** BeginHeader opt_init */
void opt_init(void);
/*** EndHeader  */

void opt_init(void)
{

   opt.measure_interval = 600;
   opt.k = 1;
   opt.q = 0;
   printf("opt_init\n") ;
   log_add("opt_init OK\n");
   opt_save();
}


/*** BeginHeader format */
void format(void);
/*** EndHeader  */

void format(void)
{
	long part_free_size;
   int part_free_size_kB;
	int dev,i,rc;

   dev=0;
   i=0; // partition A

   printf("Unmounting Partition %c . . . \n",
                                   'A' + i );
		   rc = fat_UnmountPartition(fat_part_mounted[i]);
		   if (rc) {
			   printf("fat_UnmountPartition Error: (%d) %ls\n\n", rc,
                             error_message(rc));

		   }

   rc = fat_FormatPartition( &(_fat_device_table[dev]),
											   fat_part_mounted[i], i, 6, NULL, NULL);
   if (rc) {
  			printf("fat_FormatPartition Error: (%d) %ls\n\n", rc,
                          error_message(rc));

	}

   rc = fat_AutoMount(FDDF_USE_DEFAULT);
   partition = NULL;
	for (i = 0; i < num_fat_devices * FAT_MAX_PARTITIONS; ++i)
   {
		if ((partition = fat_part_mounted[i]) != NULL)
      {
			break; // found a mounted partition, so use it
		}
	}
   if(partition == NULL)
   	rc= (rc < 0) ? rc : -ENOPART;
   else
   {
    	printf("fat_AutoMount() succeeded with return code %d.\n", rc);
      rc=0;
   }
	if (rc < 0)
   {
   	if (rc == -EUNFORMAT)
      	printf("Device not Formatted, Please run Fmt_Device.c\n");
      else
	   	printf("fat_AutoMount() failed with return code %d.\n", rc);
   }
   part_free_size = fat_Free(partition); // proc to nejede na jednom radku???
   part_free_size *= FAT_SECSIZE;
   part_free_size_kB = (int)(part_free_size/1000.0) ;


   if(fix_err == 1)
	   log_add(&last_err);
   fix_err = 0; //error fixed
   log_add("device A format OK\n");
   sprintf(&last_err, "Na disku je: %d kB volneho mista\n\n", part_free_size_kB);
   printf(&last_err);
   log_add(&last_err);
}


/*** BeginHeader log_add */
void log_add(char *text);
/*** EndHeader  */

void log_add(char *text)
{
   FATfile log_file;  // fat file structure
   int prealloc, rc;
   tm_typ time;
   char time_buf[25];
   long part_free_size;
   int part_free_size_kB;

   part_free_size = fat_Free(partition); // proc to nejede na jednom radku???
   part_free_size *= FAT_SECSIZE;
   part_free_size_kB = (int)(part_free_size/1000.0) ;
   if(part_free_size < 1000)
   {
   	rc = fat_Delete(partition, FAT_FILE, "LOG_MAR.TXT");
      if (rc < 0) {
   		printf("fat_Delete() in log_add failed with return code %d\n", rc);
      	return;
   	}
   }

   prealloc=0;
   rc = fat_Open( partition,	// First partition pointer from fat_AutoMount()
                 "LOG_MAR.TXT",	// Name of file.  Always an absolute path name.
                 FAT_FILE,		// Type of object, i.e. a file.
                 FAT_CREATE,	// Create the file if it does not exist.
                 &log_file,		// Fill in this structure with file details
                 &prealloc		// Number of bytes to allocate.
                );
	if (rc < 0) {
   	printf("fat_Open() failed with return code %d\n", rc);
		return;
   }
   rc = fat_Seek (&log_file, 0, SEEK_END);
   if (rc < 0) {
   	printf("fat_Seek() failed with return code %d\n", rc);
   }

   mktm(&time, SEC_TIMER);
   sprintf(&time_buf, "%d.%d.%d %d:%d:%d: ", time.tm_mday, time.tm_mon,
   time.tm_year+1900, time.tm_hour, time.tm_min, time.tm_sec);
   rc = fat_Write( &log_file, &time_buf, strlen(&time_buf) );
   if (rc < 0) {
   	printf("fat_Write() failed with return code %d\n", rc);
      return;
   }

   rc = fat_Write( &log_file, text, strlen(text) );
   if (rc < 0) {
   	printf("fat_Write() failed with return code %d\n", rc);
      return;
   }
  	rc = fat_Close(&log_file);
   if (rc < 0) {
   	printf("fat_Close() failed with return code %d\n", rc);
		return;
   }
}



/*** BeginHeader */
#endif
/*** EndHeader */

